/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "g_local.h"

/*QUAKED target_temp_entity (1 0 0) (-8 -8 -8) (8 8 8)
Fire an origin based temp entity event to the clients.
"style"		type byte
*/
void Use_Target_Tent (edict_t *ent, edict_t *other, edict_t *activator)
{
	gi.WriteByte (svc_temp_entity);
	gi.WriteByte (ent->style);
	gi.WritePosition (ent->s.origin);
	gi.multicast (ent->s.origin, MULTICAST_PVS);
}

void SP_target_temp_entity (edict_t *ent)
{
	ent->use = Use_Target_Tent;
}


//==========================================================

//==========================================================

/*QUAKED target_speaker (1 0 0) (-8 -8 -8) (8 8 8) looped-on looped-off reliable
"noise"		wav file to play
"attenuation"
-1 = none, send to whole level
1 = normal fighting sounds
2 = idle sound level
3 = ambient sound level
"volume"	0.0 to 1.0

Normal sounds play each time the target is used.  The reliable flag can be set for crucial voiceovers.

Looped sounds are always atten 3 / vol 1, and the use function toggles it on/off.
Multiple identical looping sounds will just increase volume without any speed cost.
*/
void Use_Target_Speaker (edict_t *ent, edict_t *other, edict_t *activator)
{
	int		chan;

	if (ent->spawnflags & 3)
	{	// looping sound toggles
		if (ent->s.sound)
			ent->s.sound = 0;	// turn it off
		else
			ent->s.sound = ent->noise_index;	// start it
	}
	else
	{	// normal sound
		if (ent->spawnflags & 4)
			chan = CHAN_VOICE|CHAN_RELIABLE;
		else
			chan = CHAN_VOICE;
		// use a positioned_sound, because this entity won't normally be
		// sent to any clients because it is invisible
		gi.positioned_sound (ent->s.origin, ent, chan, ent->noise_index, ent->volume, ent->attenuation, 0);
	}
}

void SP_target_speaker (edict_t *ent)
{
	char	buffer[MAX_QPATH];

	if(!st.noise)
	{
		gi.dprintf("target_speaker with no noise set at %s\n", vtos(ent->s.origin));
		return;
	}
	if (!strstr (st.noise, ".wav"))
		Com_sprintf (buffer, sizeof(buffer), "%s.wav", st.noise);
	else
		strncpy (buffer, st.noise, sizeof(buffer));
	ent->noise_index = gi.soundindex (buffer);

	if (!ent->volume)
		ent->volume = 1.0;

	if (!ent->attenuation)
		ent->attenuation = 1.0;
	else if (ent->attenuation == -1)	// use -1 so 0 defaults to 1
		ent->attenuation = 0;

	// check for prestarted looping sound
	if (ent->spawnflags & 1)
		ent->s.sound = ent->noise_index;

	ent->use = Use_Target_Speaker;

	// must link the entity so we get areas and clusters so
	// the server can determine who to send updates to
	gi.linkentity (ent);
}


//==========================================================

void Use_Target_Help (edict_t *ent, edict_t *other, edict_t *activator)
{
        /*        
	if (ent->spawnflags & 1)
		strncpy (game.helpmessage1, ent->message, sizeof(game.helpmessage2)-1);
	else
		strncpy (game.helpmessage2, ent->message, sizeof(game.helpmessage1)-1);
        */
        strncpy (game.helpmessage1, ent->message, sizeof(game.helpmessage2)-1);

        //ent->client->ps.pmove.pm_type = PM_FREEZE;
        //activator->client->ps.pmove.pm_type = PM_FREEZE;

        game.helpchanged++;
        Cmd_Help_f (activator);
        //
}

/*QUAKED target_help (1 0 1) (-16 -16 -24) (16 16 24) help1
When fired, the "message" key becomes the current personal computer string, and the message light will be set on all clients status bars.
*/
void SP_target_help(edict_t *ent)
{
	if (deathmatch->value)
	{	// auto-remove for deathmatch
		G_FreeEdict (ent);
		return;
	}

        /*
	if (!ent->message)
	{
		gi.dprintf ("%s with no message at %s\n", ent->classname, vtos(ent->s.origin));
		G_FreeEdict (ent);
		return;
	}
        */

	ent->use = Use_Target_Help;
}

//==========================================================

/*QUAKED target_secret (1 0 1) (-8 -8 -8) (8 8 8)
Counts a secret found.
These are single use targets.
*/
void use_target_secret (edict_t *ent, edict_t *other, edict_t *activator)
{
	gi.sound (ent, CHAN_VOICE, ent->noise_index, 1, ATTN_NORM, 0);

	level.found_secrets++;

	G_UseTargets (ent, activator);
	G_FreeEdict (ent);
}

void SP_target_secret (edict_t *ent)
{
	if (deathmatch->value)
	{	// auto-remove for deathmatch
		G_FreeEdict (ent);
		return;
	}

	ent->use = use_target_secret;
	if (!st.noise)
		st.noise = "misc/secret.wav";
	ent->noise_index = gi.soundindex (st.noise);
	ent->svflags = SVF_NOCLIENT;
	level.total_secrets++;
	// map bug hack
	if (!Q_stricmp(level.mapname, "mine3") && ent->s.origin[0] == 280 && ent->s.origin[1] == -2048 && ent->s.origin[2] == -624)
		ent->message = "You have found a secret area.";
}

//==========================================================

/*QUAKED target_goal (1 0 1) (-8 -8 -8) (8 8 8)
Counts a goal completed.
These are single use targets.
*/
void use_target_goal (edict_t *ent, edict_t *other, edict_t *activator)
{
	gi.sound (ent, CHAN_VOICE, ent->noise_index, 1, ATTN_NORM, 0);

	level.found_goals++;

	if (level.found_goals == level.total_goals)
		gi.configstring (CS_CDTRACK, "0");

	G_UseTargets (ent, activator);
	G_FreeEdict (ent);
}

void SP_target_goal (edict_t *ent)
{
	if (deathmatch->value)
	{	// auto-remove for deathmatch
		G_FreeEdict (ent);
		return;
	}

	ent->use = use_target_goal;
	if (!st.noise)
		st.noise = "misc/secret.wav";
	ent->noise_index = gi.soundindex (st.noise);
	ent->svflags = SVF_NOCLIENT;
	level.total_goals++;
}

//==========================================================


/*QUAKED target_explosion (1 0 0) (-8 -8 -8) (8 8 8)
Spawns an explosion temporary entity when used.

"delay"		wait this long before going off
"dmg"		how much radius damage should be done, defaults to 0
*/
void target_explosion_explode (edict_t *self)
{
	float		save;

	gi.WriteByte (svc_temp_entity);
	gi.WriteByte (TE_EXPLOSION1);
	gi.WritePosition (self->s.origin);
	gi.multicast (self->s.origin, MULTICAST_PHS);

        //T_RadiusDamage (self, self->activator, self->dmg, NULL, self->dmg+40, MOD_EXPLOSIVE);

	save = self->delay;
	self->delay = 0;
	G_UseTargets (self, self->activator);
	self->delay = save;
}

void use_target_explosion (edict_t *self, edict_t *other, edict_t *activator)
{
	self->activator = activator;

	if (!self->delay)
	{
		target_explosion_explode (self);
		return;
	}

	self->think = target_explosion_explode;
	self->nextthink = level.time + self->delay;
}

void SP_target_explosion (edict_t *ent)
{
        
	ent->use = use_target_explosion;
	ent->svflags = SVF_NOCLIENT;
}


//==========================================================

/*QUAKED target_changelevel (1 0 0) (-8 -8 -8) (8 8 8)
Changes level to "map" when fired
*/
void use_target_changelevel (edict_t *self, edict_t *other, edict_t *activator)
{
	if (level.intermissiontime)
		return;		// already activated

	if (!deathmatch->value && !coop->value)
	{
		if (g_edicts[1].health <= 0)
			return;
	}

	// if noexit, do a ton of damage to other
	if (deathmatch->value && !( (int)dmflags->value & DF_ALLOW_EXIT) && other != world)
	{
		T_Damage (other, self, self, vec3_origin, other->s.origin, vec3_origin, 10 * other->max_health, 1000, 0, MOD_EXIT);
		return;
	}

	// if multiplayer, let everyone know who hit the exit
	if (deathmatch->value)
	{
		if (activator && activator->client)
			gi.bprintf (PRINT_HIGH, "%s exited the level.\n", activator->client->pers.netname);
	}

	// if going to a new unit, clear cross triggers
	if (strstr(self->map, "*"))
        {
		game.serverflags &= ~(SFL_CROSS_TRIGGER_MASK);
		if(activator->client)
		{
			activator->client->pers.spawn_landmark = false;
			activator->client->pers.spawn_levelchange = false;
		}
        }
        else
	{
                if((self->spawnflags & 2) && activator->client)
		{
			activator->client->pers.spawn_landmark = true;
			VectorSubtract(activator->s.origin,self->s.origin,
				activator->client->pers.spawn_offset);
			VectorCopy(activator->velocity,activator->client->pers.spawn_velocity);
			VectorCopy(activator->s.angles,activator->client->pers.spawn_angles);
			activator->client->pers.spawn_angles[ROLL] = 0;
			VectorCopy(activator->client->ps.viewangles,activator->client->pers.spawn_viewangles);
			activator->client->pers.spawn_pm_flags = activator->client->ps.pmove.pm_flags;


                        if(self->s.angles[YAW])
			{
				vec3_t	angles;
				vec3_t	forward, right, v;

				angles[PITCH] = angles[ROLL] = 0.;
				angles[YAW] = self->s.angles[YAW];
				AngleVectors(angles,forward,right,NULL);
				VectorNegate(right,right);
				VectorCopy(activator->client->pers.spawn_offset,v);
				G_ProjectSource (vec3_origin,
					             v, forward, right,
								 activator->client->pers.spawn_offset);
				VectorCopy(activator->client->pers.spawn_velocity,v);
				G_ProjectSource (vec3_origin,
					             v, forward, right,
								 activator->client->pers.spawn_velocity);
				activator->client->pers.spawn_angles[YAW] += angles[YAW];
				activator->client->pers.spawn_viewangles[YAW] += angles[YAW];
                                
			}

		}		
		else
		{
			activator->client->pers.spawn_landmark = false;
		}

		activator->client->pers.spawn_levelchange = true;
                activator->client->pers.spawn_gunframe    = activator->client->ps.gunframe;
		activator->client->pers.spawn_modelframe  = activator->s.frame;
		activator->client->pers.spawn_anim_end    = activator->client->anim_end;
	}


	BeginIntermission (self);
}

void SP_target_changelevel (edict_t *ent)
{
	if (!ent->map)
	{
		gi.dprintf("target_changelevel with no map at %s\n", vtos(ent->s.origin));
		G_FreeEdict (ent);
		return;
	}


	if ((deathmatch->value || coop->value) && (ent->spawnflags & 2))
	{
		gi.dprintf("target_changelevel at %s\nLANDMARK only valid in single-player\n",
			vtos(ent->s.origin));
		ent->spawnflags &= ~2;
	}


	// ugly hack because *SOMEBODY* screwed up their map
   if((Q_stricmp(level.mapname, "fact1") == 0) && (Q_stricmp(ent->map, "fact3") == 0))
	   ent->map = "fact3$secret1";

	ent->use = use_target_changelevel;
	ent->svflags = SVF_NOCLIENT;
}


//==========================================================

/*QUAKED target_splash (1 0 0) (-8 -8 -8) (8 8 8)
Creates a particle splash effect when used.

Set "sounds" to one of the following:
  1) sparks
  2) blue water
  3) brown water
  4) slime
  5) lava
  6) blood

"count"	how many pixels in the splash
"dmg"	if set, does a radius damage at this location when it splashes
		useful for lava/sparks
*/

void use_target_splash (edict_t *self, edict_t *other, edict_t *activator)
{
	gi.WriteByte (svc_temp_entity);
	gi.WriteByte (TE_SPLASH);
	gi.WriteByte (self->count);
	gi.WritePosition (self->s.origin);
	gi.WriteDir (self->movedir);
	gi.WriteByte (self->sounds);
	gi.multicast (self->s.origin, MULTICAST_PVS);

	if (self->dmg)
		T_RadiusDamage (self, activator, self->dmg, NULL, self->dmg+40, MOD_SPLASH);
}

void SP_target_splash (edict_t *self)
{
	self->use = use_target_splash;
	G_SetMovedir (self->s.angles, self->movedir);

	if (!self->count)
		self->count = 32;

	self->svflags = SVF_NOCLIENT;
}


//==========================================================

/*QUAKED target_spawner (1 0 0) (-8 -8 -8) (8 8 8)
Set target to the type of entity you want spawned.
Useful for spawning monsters and gibs in the factory levels.

For monsters:
	Set direction to the facing you want it to have.

For gibs:
	Set direction if you want it moving and
	speed how fast it should be moving otherwise it
	will just be dropped
*/
void ED_CallSpawn (edict_t *ent);

void use_target_spawner (edict_t *self, edict_t *other, edict_t *activator)
{
	edict_t	*ent;

	ent = G_Spawn();
	ent->classname = self->target;
	VectorCopy (self->s.origin, ent->s.origin);
	VectorCopy (self->s.angles, ent->s.angles);
	ED_CallSpawn (ent);
	gi.unlinkentity (ent);
	KillBox (ent);
	gi.linkentity (ent);
	if (self->speed)
		VectorCopy (self->movedir, ent->velocity);
}

void SP_target_spawner (edict_t *self)
{
	self->use = use_target_spawner;
	self->svflags = SVF_NOCLIENT;
	if (self->speed)
	{
		G_SetMovedir (self->s.angles, self->movedir);
		VectorScale (self->movedir, self->speed, self->movedir);
	}
}

//==========================================================

/*QUAKED target_blaster (1 0 0) (-8 -8 -8) (8 8 8) NOTRAIL NOEFFECTS
Fires a blaster bolt in the set direction when triggered.

dmg		default is 15
speed	default is 1000
*/

void use_target_blaster (edict_t *self, edict_t *other, edict_t *activator)
{
	int effect;

	if (self->spawnflags & 2)
		effect = 0;
	else if (self->spawnflags & 1)
		effect = EF_HYPERBLASTER;
	else
		effect = EF_BLASTER;

	fire_blaster (self, self->s.origin, self->movedir, self->dmg, self->speed, EF_BLASTER, MOD_TARGET_BLASTER);
	gi.sound (self, CHAN_VOICE, self->noise_index, 1, ATTN_NORM, 0);
}

void SP_target_blaster (edict_t *self)
{
	self->use = use_target_blaster;
	G_SetMovedir (self->s.angles, self->movedir);
	self->noise_index = gi.soundindex ("weapons/laser2.wav");

	if (!self->dmg)
		self->dmg = 15;
	if (!self->speed)
		self->speed = 1000;

	self->svflags = SVF_NOCLIENT;
}


//==========================================================

/*QUAKED target_crosslevel_trigger (.5 .5 .5) (-8 -8 -8) (8 8 8) trigger1 trigger2 trigger3 trigger4 trigger5 trigger6 trigger7 trigger8
Once this trigger is touched/used, any trigger_crosslevel_target with the same trigger number is automatically used when a level is started within the same unit.  It is OK to check multiple triggers.  Message, delay, target, and killtarget also work.
*/
void trigger_crosslevel_trigger_use (edict_t *self, edict_t *other, edict_t *activator)
{
	game.serverflags |= self->spawnflags;
	G_FreeEdict (self);
}

void SP_target_crosslevel_trigger (edict_t *self)
{
	self->svflags = SVF_NOCLIENT;
	self->use = trigger_crosslevel_trigger_use;
}

/*QUAKED target_crosslevel_target (.5 .5 .5) (-8 -8 -8) (8 8 8) trigger1 trigger2 trigger3 trigger4 trigger5 trigger6 trigger7 trigger8
Triggered by a trigger_crosslevel elsewhere within a unit.  If multiple triggers are checked, all must be true.  Delay, target and
killtarget also work.

"delay"		delay before using targets if the trigger has been activated (default 1)
*/
void target_crosslevel_target_think (edict_t *self)
{
	if (self->spawnflags == (game.serverflags & SFL_CROSS_TRIGGER_MASK & self->spawnflags))
	{
		G_UseTargets (self, self);
		G_FreeEdict (self);
	}
}

void SP_target_crosslevel_target (edict_t *self)
{
	if (! self->delay)
		self->delay = 1;
	self->svflags = SVF_NOCLIENT;

	self->think = target_crosslevel_target_think;
	self->nextthink = level.time + self->delay;
}

//==========================================================

/*QUAKED target_laser (0 .5 .8) (-8 -8 -8) (8 8 8) START_ON RED GREEN BLUE YELLOW ORANGE FAT
When triggered, fires a laser.  You can either set a target
or a direction.
*/

void target_laser_think (edict_t *self)
{
	edict_t	*ignore;
	vec3_t	start;
	vec3_t	end;
	trace_t	tr;
	vec3_t	point;
	vec3_t	last_movedir;
	int		count;

	if (self->spawnflags & 0x80000000)
		count = 8;
	else
		count = 4;

	if (self->enemy)
	{
		VectorCopy (self->movedir, last_movedir);
		VectorMA (self->enemy->absmin, 0.5, self->enemy->size, point);
		VectorSubtract (point, self->s.origin, self->movedir);
		VectorNormalize (self->movedir);
		if (!VectorCompare(self->movedir, last_movedir))
			self->spawnflags |= 0x80000000;
	}

	ignore = self;
	VectorCopy (self->s.origin, start);
	VectorMA (start, 2048, self->movedir, end);
	while(1)
	{
		tr = gi.trace (start, NULL, NULL, end, ignore, CONTENTS_SOLID|CONTENTS_MONSTER|CONTENTS_DEADMONSTER);

		if (!tr.ent)
			break;

		// hurt it if we can
		if ((tr.ent->takedamage) && !(tr.ent->flags & FL_IMMUNE_LASER))
			T_Damage (tr.ent, self, self->activator, self->movedir, tr.endpos, vec3_origin, self->dmg, 1, DAMAGE_ENERGY, MOD_TARGET_LASER);

		// if we hit something that's not a monster or player or is immune to lasers, we're done
		if (!(tr.ent->svflags & SVF_MONSTER) && (!tr.ent->client))
		{
			if (self->spawnflags & 0x80000000)
			{
				self->spawnflags &= ~0x80000000;
				gi.WriteByte (svc_temp_entity);
				gi.WriteByte (TE_LASER_SPARKS);
				gi.WriteByte (count);
				gi.WritePosition (tr.endpos);
				gi.WriteDir (tr.plane.normal);
				gi.WriteByte (self->s.skinnum);
				gi.multicast (tr.endpos, MULTICAST_PVS);
			}
			break;
		}

		ignore = tr.ent;
		VectorCopy (tr.endpos, start);
	}

	VectorCopy (tr.endpos, self->s.old_origin);

	self->nextthink = level.time + FRAMETIME;
}

void target_laser_on (edict_t *self)
{
	if (!self->activator)
		self->activator = self;
	self->spawnflags |= 0x80000001;
	self->svflags &= ~SVF_NOCLIENT;
	target_laser_think (self);
}

void target_laser_off (edict_t *self)
{
	self->spawnflags &= ~1;
	self->svflags |= SVF_NOCLIENT;
	self->nextthink = 0;
}

void target_laser_use (edict_t *self, edict_t *other, edict_t *activator)
{
	self->activator = activator;
	if (self->spawnflags & 1)
		target_laser_off (self);
	else
		target_laser_on (self);
}

void target_laser_start (edict_t *self)
{
	edict_t *ent;

	self->movetype = MOVETYPE_NONE;
	self->solid = SOLID_NOT;
	self->s.renderfx |= RF_BEAM|RF_TRANSLUCENT;
	self->s.modelindex = 1;			// must be non-zero

	// set the beam diameter
	if (self->spawnflags & 64)
		self->s.frame = 16;
	else
		self->s.frame = 4;

	// set the color
	if (self->spawnflags & 2)
		self->s.skinnum = 0xf2f2f0f0;
	else if (self->spawnflags & 4)
		self->s.skinnum = 0xd0d1d2d3;
	else if (self->spawnflags & 8)
		self->s.skinnum = 0xf3f3f1f1;
	else if (self->spawnflags & 16)
		self->s.skinnum = 0xdcdddedf;
	else if (self->spawnflags & 32)
		self->s.skinnum = 0xe0e1e2e3;

	if (!self->enemy)
	{
		if (self->target)
		{
			ent = G_Find (NULL, FOFS(targetname), self->target);
			if (!ent)
				gi.dprintf ("%s at %s: %s is a bad target\n", self->classname, vtos(self->s.origin), self->target);
			self->enemy = ent;
		}
		else
		{
			G_SetMovedir (self->s.angles, self->movedir);
		}
	}
	self->use = target_laser_use;
	self->think = target_laser_think;

	if (!self->dmg)
		self->dmg = 1;

	VectorSet (self->mins, -8, -8, -8);
	VectorSet (self->maxs, 8, 8, 8);
	gi.linkentity (self);

	if (self->spawnflags & 1)
		target_laser_on (self);
	else
		target_laser_off (self);
}

void SP_target_laser (edict_t *self)
{
	// let everything else get spawned before we start firing
	self->think = target_laser_start;
	self->nextthink = level.time + 1;
}

//==========================================================

/*QUAKED target_lightramp (0 .5 .8) (-8 -8 -8) (8 8 8) TOGGLE
speed		How many seconds the ramping will take
message		two letters; starting lightlevel and ending lightlevel
*/

void target_lightramp_think (edict_t *self)
{
	char	style[2];

	style[0] = 'a' + self->movedir[0] + (level.time - self->timestamp) / FRAMETIME * self->movedir[2];
	style[1] = 0;
	gi.configstring (CS_LIGHTS+self->enemy->style, style);

	if ((level.time - self->timestamp) < self->speed)
	{
		self->nextthink = level.time + FRAMETIME;
	}
	else if (self->spawnflags & 1)
	{
		char	temp;

		temp = self->movedir[0];
		self->movedir[0] = self->movedir[1];
		self->movedir[1] = temp;
		self->movedir[2] *= -1;
	}
}

void target_lightramp_use (edict_t *self, edict_t *other, edict_t *activator)
{
	if (!self->enemy)
	{
		edict_t		*e;

		// check all the targets
		e = NULL;
		while (1)
		{
			e = G_Find (e, FOFS(targetname), self->target);
			if (!e)
				break;
			if (strcmp(e->classname, "light") != 0)
			{
				gi.dprintf("%s at %s ", self->classname, vtos(self->s.origin));
				gi.dprintf("target %s (%s at %s) is not a light\n", self->target, e->classname, vtos(e->s.origin));
			}
			else
			{
				self->enemy = e;
			}
		}

		if (!self->enemy)
		{
			gi.dprintf("%s target %s not found at %s\n", self->classname, self->target, vtos(self->s.origin));
			G_FreeEdict (self);
			return;
		}
	}

	self->timestamp = level.time;
	target_lightramp_think (self);
}

void SP_target_lightramp (edict_t *self)
{
	if (!self->message || strlen(self->message) != 2 || self->message[0] < 'a' || self->message[0] > 'z' || self->message[1] < 'a' || self->message[1] > 'z' || self->message[0] == self->message[1])
	{
		gi.dprintf("target_lightramp has bad ramp (%s) at %s\n", self->message, vtos(self->s.origin));
		G_FreeEdict (self);
		return;
	}

	if (deathmatch->value)
	{
		G_FreeEdict (self);
		return;
	}

	if (!self->target)
	{
		gi.dprintf("%s with no target at %s\n", self->classname, vtos(self->s.origin));
		G_FreeEdict (self);
		return;
	}

	self->svflags |= SVF_NOCLIENT;
	self->use = target_lightramp_use;
	self->think = target_lightramp_think;

	self->movedir[0] = self->message[0] - 'a';
	self->movedir[1] = self->message[1] - 'a';
	self->movedir[2] = (self->movedir[1] - self->movedir[0]) / (self->speed / FRAMETIME);
}

//==========================================================

/*QUAKED target_earthquake (1 0 0) (-8 -8 -8) (8 8 8)
When triggered, this initiates a level-wide earthquake.
All players and monsters are affected.
"speed"		severity of the quake (default:200)
"count"		duration of the quake (default:5)
*/

void target_earthquake_think (edict_t *self)
{
	int		i;
	edict_t	*e;

        /*
        //sound fx stuff, no need for it.
	if (self->last_move_time < level.time)
	{
                //gi.positioned_sound (self->s.origin, self, CHAN_AUTO, self->noise_index, 1.0, ATTN_NONE, 0);
		self->last_move_time = level.time + 0.5;
	}
        */

	for (i=1, e=g_edicts+i; i < globals.num_edicts; i++,e++)
	{
		if (!e->inuse)
			continue;
		if (!e->client)
			continue;
		if (!e->groundentity)
			continue;

		e->groundentity = NULL;
                e->velocity[0] += crandom()* 64;//150
                e->velocity[1] += crandom()* 64;
		e->velocity[2] = self->speed * (100.0 / e->mass);
	}

	if (level.time < self->timestamp)
		self->nextthink = level.time + FRAMETIME;
}

void target_earthquake_use (edict_t *self, edict_t *other, edict_t *activator)
{
	self->timestamp = level.time + self->count;
	self->nextthink = level.time + FRAMETIME;
	self->activator = activator;
	self->last_move_time = 0;
}

void SP_target_earthquake (edict_t *self)
{
	if (!self->targetname)
        {
		gi.dprintf("untargeted %s at %s\n", self->classname, vtos(self->s.origin));
                G_FreeEdict(self);
		return;
        }

        /*
	if (!self->count)
		self->count = 5;
        */

	if (!self->speed)
		self->speed = 200;

	self->svflags |= SVF_NOCLIENT;
	self->think = target_earthquake_think;
	self->use = target_earthquake_use;

        //self->noise_index = gi.soundindex ("world/quake.wav");
}

//======================== TARGET_SHOOTER ===================================
//target_shooter: a general "particle" shooter
//based on lazarus target_shooter

#define SF_STARTON 1
#define SF_LOOP 2
#define SF_NOGRAVITY 4
#define SF_SMOKE 8

void directed_debris_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point)
{
	G_FreeEdict (self);
}
void FadeDieThink(edict_t *);
void ThrowRock (edict_t *self, char *modelname, float speed, vec3_t origin, vec3_t size, int mass)
{
	edict_t	*chunk;
	vec_t	var = speed/5;

	chunk = G_Spawn();
	VectorCopy (origin, chunk->s.origin);
	gi.setmodel (chunk, modelname);
	VectorCopy(size,chunk->maxs);
	VectorScale(chunk->maxs,0.5,chunk->maxs);
	VectorNegate(chunk->maxs,chunk->mins);

        //
        chunk->s.renderfx |= RF_TRANSLUCENT;
        //chunk->s.effects |= EF_SPHERETRANS;

        if (self->spawnflags & SF_SMOKE)
                chunk->s.effects |= EF_GRENADE;

        chunk->count = 254;

	chunk->velocity[0] = speed * self->movedir[0] + var * crandom();
	chunk->velocity[1] = speed * self->movedir[1] + var * crandom();
	chunk->velocity[2] = speed * self->movedir[2] + var * crandom();

        if (self->spawnflags & SF_NOGRAVITY)
                chunk->movetype = MOVETYPE_FLY;
        else
                chunk->movetype = MOVETYPE_TOSS;

	chunk->attenuation = 0.5;
	chunk->solid = SOLID_NOT;
        chunk->avelocity[0] = random()*600;
        chunk->avelocity[1] = random()*600;
	chunk->avelocity[2] = random()*600;
	chunk->think = FadeDieThink;

        //chunk->nextthink = level.time + random()*5;

        //
        chunk->nextthink = level.time + (random() * self->wait);

	chunk->s.frame = 0;
	chunk->flags = 0;
	chunk->classname = "debris";
	chunk->takedamage = DAMAGE_YES;
	chunk->die = directed_debris_die;
	chunk->mass = mass;
	gi.linkentity (chunk);
}

void use_target_shooter (edict_t *self, edict_t *other, edict_t *activator)
{
	vec3_t	chunkorigin;
	vec3_t	size, source;
	vec_t	mass;
	int		count;
	char	modelname[64];

	VectorSet(source,8,8,8);
	mass = self->mass;
	// big chunks
        // todo: simplify this section
	if (mass >= 100)
	{
                if(strstr(self->usermodel,".sp2"))
                        sprintf(modelname, "sprites/%s", self->usermodel);
                else
                        sprintf(modelname, "models/%s", self->usermodel);


                //sprintf(modelname, self->model);
		count = mass / 100;
		if (count > 16)
			count = 16;
		VectorSet(size,8,8,8);
		while(count--)
		{
			chunkorigin[0] = self->s.origin[0] + crandom() * source[0];
			chunkorigin[1] = self->s.origin[1] + crandom() * source[1];
			chunkorigin[2] = self->s.origin[2] + crandom() * source[2];
                        ThrowRock (self, modelname, self->speed, chunkorigin, size, 100);
		}
	}

	self->count--;
        if(!self->count)
        {
		self->think = G_FreeEdict;
		self->nextthink = level.time + 1;
	}


        if (self->spawnflags & SF_LOOP)
        {
                self->think = use_target_shooter;
                self->nextthink = level.time + 0.1;
        }
}

void SP_target_shooter (edict_t *self)
{
	//paranoia checks
        if (!self->usermodel)
	{
                gi.dprintf("%s without a model at %s\n", self->classname, vtos(self->s.origin));
                G_FreeEdict(self);
		return;
	}


        self->class_id = ENTITY_TARGET_SHOOTER;
        //gi.modelindex ("sprites/drop001.sp2");

        char    modelname[256];

        if(strstr(self->usermodel,".sp2"))
                sprintf(modelname, "sprites/%s", self->usermodel);
        else
                sprintf(modelname, "models/%s", self->usermodel);

        //strcpy (self->model, modelname);

        //self->model = modelname;
        self->s.modelindex = gi.modelindex (modelname);


	if(!self->speed)
		self->speed = 400;
	if (!self->mass)
		self->mass = 500;
        self->use = use_target_shooter;
	G_SetMovedir (self->s.angles, self->movedir);
	self->svflags = SVF_NOCLIENT;

        if (self->spawnflags & SF_STARTON)
        {
                self->think = use_target_shooter;
                self->nextthink = level.time + 0.1;
        }
}

//============================= TARGET_FADE =================================
void use_target_fade (edict_t *self, edict_t *other, edict_t *activator)
{
	if(!activator->client)
		return;

	activator->client->fadestart= level.framenum;
	activator->client->fadein   = level.framenum + self->fadein*10;
	activator->client->fadehold = activator->client->fadein + self->holdtime*10;
	activator->client->fadeout  = activator->client->fadehold + self->fadeout*10;
        activator->client->fadecolor[0] = self->color[0];
        activator->client->fadecolor[1] = self->color[1];
        activator->client->fadecolor[2] = self->color[2];
	activator->client->fadealpha    = self->alpha;

	self->count--;
	if(!self->count) {
		self->think = G_FreeEdict;
		self->nextthink = level.time + 1;
	}
}

void SP_target_fade (edict_t *self)
{
	self->class_id = ENTITY_TARGET_FADE;
	self->use = use_target_fade;

	if(self->fadein < 0)
		self->fadein = 0;

	if(self->holdtime < 0)
	{
		self->count = 1;
		self->holdtime = 10000;
	}

	if(self->fadeout < 0)
		self->fadeout = 0;
}


// ================================ TARGET_VIEWLOCK ========================
//lock player view to 10-170 degree range. refer to client/cl_input.c
#define SF_LOCKOFF      1
void use_target_viewlock (edict_t *self, edict_t *other, edict_t *activator)
{
        if (self->spawnflags & SF_LOCKOFF) //turn off!
                activator->client->ps.rdflags &= ~RDF_NOCLEAR;
        else //turn on
                activator->client->ps.rdflags |=  RDF_NOCLEAR;
}

void SP_target_viewlock (edict_t *self)
{
        self->class_id = ENTITY_TARGET_VIEWLOCK;
        self->use = use_target_viewlock;
}



//============================= TARGET_DEBRIS ================================
// code so very messy.
#define SF_STARTON      1
#define SF_SUPERSPEW    1

void spacedebris_touch (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf)
{
        

	if (surf && (surf->flags & SURF_SKY))
	{
                G_FreeEdict (ent);
	}
        else
                return;


}

void debris_throw (edict_t *self)
{
	edict_t	*chunk;

	chunk = G_Spawn();

        //noshadows.
        chunk->s.renderfx |= (RF_NOSHADOW | RF_FULLBRIGHT);


        //random orientation
        chunk->s.angles[0] = random() * 359;
        chunk->s.angles[1] = random() * 359;
        chunk->s.angles[2] = random() * 359;

        //random origin point
        VectorCopy (self->s.origin, chunk->s.origin);
        chunk->s.origin[0] += (self->bleft[0]*random() + self->tright[0]*random());
        chunk->s.origin[1] += (self->bleft[1]*random() + self->tright[1]*random());
        chunk->s.origin[2] += (self->bleft[2]*random() + self->tright[2]*random());

        //random speed
        int speed = (random() * 160) + 64;
        chunk->velocity[0] = speed * self->movedir[0];
        chunk->velocity[1] = speed * self->movedir[1];
        chunk->velocity[2] = speed * self->movedir[2];

        
        if (self->health == 1) //debris
        {
                //random avelocity
                chunk->avelocity[0] = (random()*256) + 256;
                //if ( ((int)(random()*2)) == 1)
                //        chunk->avelocity[1] = (random()*256) + 128;
                if ( ((int)(random()*2)) == 1)
                        chunk->avelocity[2] = (random()*256) + 128;
        }
        else
        {
                //ships
                chunk->avelocity[0] = (random()*64) + 32;
                //if ( ((int)(random()*2)) == 1)
                //        chunk->avelocity[1] = (random()*64) + 32;
                if ( ((int)(random()*2)) == 1)
                        chunk->avelocity[2] = (random()*64) + 32;
        }

        chunk->touch = spacedebris_touch;

        if (self->health == 1)
        {
                //spacedebris

                //random skin
                int iskins = 8;
                chunk->s.skinnum = (int)(random() * iskins);

                //random frame
                if ( ((int)(random()*4)) == 1)
                        chunk->s.frame = 1;
        }
        else
        {
                //random frame
                if ( ((int)(random()*3)) != 1)
                        chunk->s.frame = 1;
        }



	chunk->flags = 0;
        chunk->classname = "spacedebris";
        gi.setmodel (chunk, self->usermodel);
        chunk->movetype = MOVETYPE_FLY;
        chunk->solid = SOLID_BBOX;

	gi.linkentity (chunk);

}

void debris_think (edict_t *self)
{
        debris_throw (self);

        self->think = debris_think;

        if (self->health == 1)
        {
                if (self->spawnflags & SF_SUPERSPEW)
                        self->nextthink = level.time + 0.01;//dangerous...
                else
                        self->nextthink = level.time + 0.2;//spacedebris.
        }
        else
                self->nextthink = level.time + (int)(random() * 32) + 16;
}

void debris_use (edict_t *self)
{
        if (self->spawnflags & SF_STARTON)
        {
                //already on, therefore turn it off
                self->spawnflags &= ~SF_STARTON;
                self->nextthink = 0;
        }
        else
        {
                //off, therefore turn it on
                self->spawnflags |= SF_STARTON;

                self->think = debris_think;
                if (self->health == 1)
                        self->think(self);
                else
                        self->nextthink = level.time + (int)(random() * 32) + 4;
        }
}

void SP_target_debris (edict_t *self)
{
        //paranoia check
        if ( !(self->spawnflags & SF_STARTON) && !(self->targetname))
	{
                gi.dprintf("%s error at %s\n", self->classname, vtos(self->s.origin));
                G_FreeEdict(self);
		return;
	}

        

        switch (self->health)
        {
                default:        
                case 1:
                        self->s.modelindex = gi.modelindex("models/space/debris/tris.md2");
                        self->usermodel    = "models/space/debris/tris.md2";
                        break;
                case 2:
                        self->s.modelindex = gi.modelindex("models/space/tie/tris.md2");
                        self->usermodel    = "models/space/tie/tris.md2";
                        break;
        }

        self->svflags   |= SVF_NOCLIENT;

        if (self->spawnflags & SF_STARTON) //automatically start.
        {
                self->spawnflags &= ~SF_STARTON;
                self->think = debris_use;
                self->nextthink = level.time + 1;
        }

        
        G_SetMovedir (self->s.angles, self->movedir);
        self->use = debris_use;

        gi.linkentity (self);
}



/*=============================================================================
 TARGET_SKY - uses the "sky" string as parameter to "sky" console command.
 Best used in areas where player cannot see the sky, of course.

 NOTE: Tried using a START_ON spawnflag, but it ends up being meaningless.
       If we don't delay use_target_sky a bit at level start, we get
       "SZ_GetSpace: overflow without allowoverflow set", and if we use
       a sufficient delay to get around the error then the original sky
       is visible before the switch.
=============================================================================*/
void use_target_sky (edict_t *self, edict_t *other, edict_t *activator)
{
	gi.configstring(CS_SKY,self->pathtarget);

        //stuffcmd(&g_edicts[1],va("sky %s\n",self->pathtarget));
        gi.AddCommandString (va("sky %s\n",self->pathtarget));


        self->count--;
	if(!self->count) {
		self->think = G_FreeEdict;
		self->nextthink = level.time + 1;
	}
}

void SP_target_sky (edict_t *self)
{
	if(!st.sky || !*st.sky)
	{
		gi.dprintf("Target_sky with no sky string at %s\n",vtos(self->s.origin));
		G_FreeEdict(self);
		return;
	}


	self->class_id = ENTITY_TARGET_SKY;
	self->pathtarget = gi.TagMalloc(strlen(st.sky)+1,TAG_LEVEL);
	strcpy(self->pathtarget,st.sky);

        //precache the skybox art so that there is no loading pause.
        //this doesnt work.
        /*
        gi.imageindex (va("../env/%sup",self->pathtarget));
        gi.imageindex (va("../env/%sdn",self->pathtarget));
        gi.imageindex (va("../env/%sft",self->pathtarget));
        gi.imageindex (va("../env/%slf",self->pathtarget));
        gi.imageindex (va("../env/%srt",self->pathtarget));
        gi.imageindex (va("../env/%sbk",self->pathtarget));
        */
        //bc
        self->svflags = SVF_NOCLIENT;
	self->use = use_target_sky;
}

//========================= TARGET_TITLE ===============================
//THIS IS a one-shot deal. use it, teh title screen pops up.use it again,
//the titlescreen disappears. easy as pie

void use_target_title (edict_t *self, edict_t *other, edict_t *activator)
{
        if (self->isopen == false)
        {
                self->isopen = true;

                //turn on the titlescreen.
                //cheap hack.

                if (self->spawnflags & 1)
                        activator->client->ps.stats[STAT_FLASHES] = 2;//theend
                else
                        activator->client->ps.stats[STAT_FLASHES] = 1;//title
                //bc
        }
        else
        {
                self->isopen = false;

                //turn off the titlescreen.
                activator->client->ps.stats[STAT_FLASHES] = 0;
        }
}

void SP_target_title (edict_t *self)
{
        self->class_id = ENTITY_TARGET_TITLE;

        //isopen determines whether the screen is on or off.
        self->isopen = false;

        //precache image.
        gi.imageindex ("title");

        //set the use function.
        self->use = use_target_title;
        self->svflags = SVF_NOCLIENT;
}


//========================= TARGET_FX ===============================
void use_target_fx (edict_t *self, edict_t *other, edict_t *activator)
{
        int     tag;
        char    command[32];

        if (self->spawnflags & 1)
                tag = 0;
        else
                tag = 1;

        switch (self->health)
        {
                case 0: sprintf(command, "gl_motionblur %i\n", tag); break;
        }

        if (command)
                gi.AddCommandString (command);
}

void SP_target_fx (edict_t *self)
{
        self->class_id = ENTITY_TARGET_FX;

        //set the use function.
        self->use = use_target_fx;
        self->svflags = SVF_NOCLIENT;
}
