/*
==============================================================================

butterfly.

==============================================================================
*/

#include "g_local.h"

#define FRAME_fly1      0
#define FRAME_fly5      1

// STAND

void butterfly_stand (edict_t *self);
void butterfly_fly (edict_t *self);
void butterfly_move_turn (edict_t *self);
void butterfly_turn (edict_t *self);
void butterfly_turn_end (edict_t *self);

int butterfly_ai_check_move_forward (edict_t *self)
{
	trace_t tr;
	vec3_t forward, end;

	AngleVectors(self->s.angles, forward, NULL, NULL);
        VectorMA(self->s.origin, 256, forward, end);//WAS128

        tr = gi.trace (self->s.origin, self->mins, self->maxs, end, self, MASK_SOLID);

	if(tr.fraction != 1)
		return 0;

	return 1;
}

mframe_t butterfly_frames_stand [] =
{
	ai_stand, 0, NULL,
	ai_stand, 0, NULL
};

mmove_t butterfly_move_stand = {FRAME_fly1, FRAME_fly5, butterfly_frames_stand, butterfly_fly};

void butterfly_stand (edict_t *self)
{
        self->monsterinfo.currentmove = &butterfly_move_stand;
}

//
// FLY
//

mframe_t butterfly_frames_turn [] =
{
        ai_move, 9,  butterfly_turn,
        ai_move, 9,  butterfly_turn_end
};
mmove_t butterfly_move_turn1 = {FRAME_fly1, FRAME_fly5, butterfly_frames_turn, butterfly_fly};

mframe_t butterfly_frames_fly [] =
{
	ai_stand, 9, NULL,
	ai_stand, 9, NULL
};
mmove_t butterfly_move_fly = {FRAME_fly1, FRAME_fly5, butterfly_frames_fly, butterfly_fly};

void butterfly_turn_end (edict_t *self)
{
        if(butterfly_ai_check_move_forward(self))
	{
		self->count = 0;
		self->s.angles[2] = 0;
	}
	else
	{
                self->monsterinfo.currentmove = &butterfly_move_turn1;
	}
}

void butterfly_turn (edict_t *self)
{
	float turn;

        turn = random()*10;//was5

	if(self->count == 1)
	{
                self->s.angles[1] += 30+turn;//was10
		self->s.angles[2] = 30;
	}
	else
	{
                self->s.angles[1] -= 30+turn;//was10
		self->s.angles[2] = -30;
	}
}

void butterfly_move_turn (edict_t *self)
{
	float n;

	n = random();

	if(n > 0.5)
	{
		self->count = 1;
		self->s.angles[2] = 15;
	}
	else
	{
		self->count = 2;
		self->s.angles[2] = -15;
	}

        self->monsterinfo.currentmove = &butterfly_move_turn1;
}

void butterfly_fly (edict_t *self)
{
	float n;

	n = random();

        if((butterfly_ai_check_move_forward (self) == 1) && (n < 0.95))
                self->monsterinfo.currentmove = &butterfly_move_fly;
	else
                butterfly_move_turn(self);
}

//
// SPAWN
//

void SP_monster_butterfly (edict_t *self)
{
	if (deathmatch->value)
	{
		G_FreeEdict (self);
		return;
	}


        int skins = 7;
        self->s.skinnum =       (int)(random()*skins);
        

        self->health =          999;
        self->max_health =      self->health;
        //self->gib_health =    -40;

        self->s.modelindex =    gi.modelindex ("models/animal/bfly1/tris.md2");
        self->monsterinfo.scale = 0.5; //speed???
        VectorSet (self->mins, -2, -2, -2);
        VectorSet (self->maxs, 2, 2, 2);

        self->movetype =        MOVETYPE_STEP;
        


        self->solid =           SOLID_NOT;
        self->yaw_speed =       24;
        self->takedamage =      DAMAGE_NO;

        self->mass = 50;

        //self->pain = butterfly_pain;
        //self->die = butterfly_die;

        self->monsterinfo.stand = butterfly_stand;
        self->monsterinfo.walk = butterfly_fly;
        self->monsterinfo.run = butterfly_fly;
	self->monsterinfo.dodge = NULL;
	self->monsterinfo.attack = NULL;
	self->monsterinfo.melee = NULL;
	self->monsterinfo.sight = NULL;

        self->monsterinfo.aiflags |= AI_GOOD_GUY;

	gi.linkentity (self);

        self->monsterinfo.stand (self);

	flymonster_start (self);
}

